// mct array driver
// mp 2003; dumarooon

module mct(ckin,mode,ad,ad4,ad4n,c0,c1,sample,gc,ckout,trgout);

`define startint   10'd998
`define stopint    10'd8
`define startdummy 10'd150
`define stopdummy  10'd160
`define muxdelay   10'd10 
`define muxtime    5'd20 
`define resettime  10'd800
`define average    6'd5
`define sensnum    5'd16

input ckin;
input [2:0] mode; // operating modes:
		  // 000 = normal gain=2
          // 001 = average gain=2
          // 010 = average gain=20
		  // 011 = background normal
          // 100 = background average gain=2
          // 101 = background average gain=20
          // 110 = pseudo boxcar
          // 111 = integrator off
output [3:0] ad;
output ad4;
output ad4n;
output c0,c1;  //integrator control
output sample; 
output gc;     //gain control 
output trgout; //start of mux sequence
output ckout;  //PLL output

//PLL divider
wire [9:0] mq;

lpm_counter pllcnt (.clock(ckin),.cout(ckout),.q(mq));
defparam pllcnt.lpm_width = 10,
         pllcnt.lpm_modulus = 1000;

//address Prescaler
wire adclk;
wire [4:0] prq;
reg adrst;

lpm_counter prcnt (.clock(ckin),.cout(adclk),.aclr(adrst),.q(prq));
defparam prcnt.lpm_width = 5,
         prcnt.lpm_modulus = `muxtime;

//address counter
wire [5:0] adq;
reg ld;
wire adcken;
wire [6:0] prset;

assign prset[5:0] = `sensnum;
assign prset[6] = 1;

lpm_counter adcnt (.clock(adclk),.clk_en(adcken),.data(prset),
					.aload(ld),.aclr(adrst),.q(adq));
defparam adcnt.lpm_width = 6;

assign ad=adq[3:0];
assign adcken=~adq[5];

//modes
reg [2:0] rmode;

always@(negedge ckin)
if(mq==(`startint-1))
	rmode=mode;

//gain
reg rgc;
always@(posedge ckin)
	case(rmode)
		2,3,5 : rgc=0;
  		0,1,4,6,7 : rgc=1;
	endcase

assign gc=rgc;

//average counter
wire avcout;
reg ensample;

lpm_counter avcnt (.clock(ckout),.cout(avcout));
defparam avcnt.lpm_width = 6,
		 avcnt.lpm_modulus = `average;

always@(negedge ckin)
	case(rmode)
		0,3,6 : ensample=1;
  		1,2,4,5 : ensample=avcout;
		7 : ensample=0;
	endcase

//phase adjustable sample
reg sample;
always@(prq) sample=(prq==12) ? (adcken & ~adrst & ensample) : 0; 

// Make P.F. happy
reg ad4n, ad4;

always@(trgout)
if((ensample & adcken & ~adrst) | (rmode==7))
	begin
		ad4n=~adq[4] & c0;
		ad4=adq[4];
	end
else
	begin
		ad4n=0;
		ad4=0;
	end


//C0, C1 assume that with 0 switch are closed
//C0=run/hold, C1=integrate/reset
reg c0,c1,c1a;
reg sig;

//trgout signal
reg trgout;

always@(negedge ckin)
	if(rmode==7)
		trgout=(mq==`stopint+`muxdelay);
	else
		trgout=adcken & ensample & (mq==`stopint+`muxdelay);

always@(posedge ckin) c1 = c1a;

always@(negedge ckin) begin
if(rmode==7)
	begin
		c0=0;
		c1a=0;
		adrst=0;
		ld=1;
	end 
else begin
ld=0;
if(sig) 
	case(mq)
//	`startint-1: c1a=0;
	`startint : begin
					c1a=1;   // integrate
					c0=0;
	                adrst=1;
				end
//	`stopint-1: c1a=0;
	`stopint : begin 
				 c0 = 1;   // hold
//				 c1a=1;
			    end
    `stopint+`muxdelay : if(ensample) adrst=0;  // start mux
	`resettime : if(ensample) 
			begin
				adrst=1;   // reset mux 
				c0 = 0;    // reset
				c1a = 0;
				case(rmode) 
					0,1,2: sig=1;
					3,4,5: sig=0;
					6: sig=~sig;
				endcase
			end
        endcase
else
	case(mq)
//		`startdummy-1: c1a=0;	
		`startdummy : begin
						c0 = 0;   // integrate
						c1a = 1;
						adrst=1;
					  end
//		`stopdummy-1: c1a=0;		
		`stopdummy : begin
						c0 = 1;   // hold
						c1a = 1;
					 end
		`stopint+`muxdelay : if(ensample) adrst=0;  // start mux
		`resettime : if(ensample) 
					 begin 
						c0 = 0;      // reset
						c1a = 0;
						case(rmode) 
							0,1,2: sig=1;
							3,4,5: sig=0;
							6: sig=~sig;
						endcase
					 end
    endcase
end

end

endmodule
